﻿////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Autodesk, Inc. All rights reserved 
// Written by Philippe Leefsma 2011 - ADN/Developer Technical Services
//
// This software is provided as is, without any warranty that it will work. You choose to use this tool at your own risk.
// Neither Autodesk nor the author Philippe Leefsma can be taken as responsible for any damage this tool can cause to 
// your data. Please always make a back up of your data prior to use this tool, as it will modify the documents involved 
// in the feature transformation.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using Inventor;

namespace ThreadModeler
{
    /////////////////////////////////////////////////////////////////
    // ThreadModeler control Dialog
    //
    /////////////////////////////////////////////////////////////////
    public partial class Controller : Form
    {
        /////////////////////////////////////////////////////////////
        // Members
        //
        /////////////////////////////////////////////////////////////
        private double _extraPitch;

        private bool _selecSetPopulated;

        private bool _extraPitchValid;

        private Inventor.Application _Application;

        private PartDocument _Document;

        private ButtonDefinition _btnDef;

        private InteractionManager _InteractionManager;

        private static string _ThreadTemplatePath = string.Empty;

        /////////////////////////////////////////////////////////////
        // Use: a small utility class to create child dialogs
        //
        /////////////////////////////////////////////////////////////
        private class WindowWrapper : 
            System.Windows.Forms.IWin32Window
        {
            private IntPtr mHwnd;

            public WindowWrapper(IntPtr handle)
            {
                mHwnd = handle;
            }

            public IntPtr Handle
            {
                get
                {
                    return mHwnd;
                }
            }
        }

        /////////////////////////////////////////////////////////////
        // Constructor
        //
        /////////////////////////////////////////////////////////////
        public Controller(Inventor.Application Application, 
            ButtonDefinition btnDef)
        {
            InitializeComponent();

            this.KeyPreview = true;

            bOk.Enabled = false;

            tbTemplate.Text = 
                (_ThreadTemplatePath == string.Empty ? 
                    FindThreadTemplate() : _ThreadTemplatePath);

            tbExtraPitch.Text = "0.10";

            tbExtraPitch.TextChanged += 
                new EventHandler(this.tbExtraPitch_TextChanged);

            lvFeatures.Items.Clear();

            _Application = Application;

            _Document = _Application.ActiveDocument as PartDocument;

            _btnDef = btnDef;

            _btnDef.Pressed = true;

            _selecSetPopulated = false;

            _extraPitch = 0.1;

            _extraPitchValid = true;

            _InteractionManager = 
                new InteractionManager(_Application);

            //Event Handlers Init

            this.FormClosed += 
                new FormClosedEventHandler(Controller_FormClosed);

            this.KeyDown += 
                new KeyEventHandler(Controller_KeyDown);

            this.tbTemplate.DoubleClick += 
                new EventHandler(tbTemplate_DoubleClick);

            _InteractionManager.OnTerminateEvent += 
                new InteractionManager.OnTerminateHandler(
                    InteractionManager_OnTerminateEvent);

            _InteractionManager.OnSelectEvent += 
                new InteractionManager.OnSelectHandler(
                    InteractionManager_OnSelectEvent);

            _InteractionManager.OnUnselectEvent += 
                new InteractionManager.OnUnselectHandler(
                    InteractionManager_OnUnselectEvent);          
        }

        /////////////////////////////////////////////////////////////
        // Use: Key down message handler. 
        //      Works with Form.Keypreview = true
        /////////////////////////////////////////////////////////////
        void Controller_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
            {
                _InteractionManager.Terminate();
            }
        }

        /////////////////////////////////////////////////////////////
        // use: Form Closed by user Handler.
        //
        /////////////////////////////////////////////////////////////
        void Controller_FormClosed(object sender, 
            FormClosedEventArgs e)
        {
            _btnDef.Pressed = false;

            _InteractionManager.Terminate();
        }

        /////////////////////////////////////////////////////////////
        // Use: OnTerminate interaction Handler.
        //
        /////////////////////////////////////////////////////////////
        void InteractionManager_OnTerminateEvent(object o, 
            InteractionManager.OnTerminateEventArgs e)
        {
            Close();
        }

        /////////////////////////////////////////////////////////////
        // Use: OnSelect ThreadFeature Handler.
        //
        /////////////////////////////////////////////////////////////
        void InteractionManager_OnSelectEvent(object o, 
            InteractionManager.OnSelectEventArgs e)
        {           
            foreach (System.Object obj in e.Entities)
            {
                ThreadFeature thread = obj as ThreadFeature;

                if (thread.Suppressed)
                    continue;

                string iMateName = string.Empty;

                if (Toolkit.HasiMate(thread.ThreadedFace[1], 
                    out iMateName))
                {
                    DialogResult res = MessageBox.Show(
                        "Threaded face or one of its edge has" +
                        " iMate associated to it." + 
                        System.Environment.NewLine +
                        "Please delete iMate " + iMateName  + 
                        " before modelizing this thread.",
                        "Invalid iMate",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);

                    continue;
                }

                double pitch = ThreadWorker.GetThreadPitch(thread);

                string pitchStr =
                    ThreadWorker.GetThreadPitchStr(thread,
                        (Document)_Document);

                string minStr =
                    _Document.UnitsOfMeasure.GetStringFromValue(
                        ThreadWorker.ThresholdPitchCm,
                        UnitsTypeEnum.kDefaultDisplayLengthUnits);

                if (pitch < ThreadWorker.ThresholdPitchCm)
                {
                    DialogResult res = MessageBox.Show(
                       "Selected thread pitch " +
                       "is too small (" + pitchStr + ")." +
                       System.Environment.NewLine +
                       "The minimum thread pitch that can " +
                       "be modelized is " + minStr + " .",
                       "Invalid Thread Pitch",
                       MessageBoxButtons.OK,
                       MessageBoxIcon.Error);

                    continue;
                }

                ListViewItem item = 
                    lvFeatures.Items.Add(thread.Name);

                item.Tag = thread;

                item.SubItems.Add(pitchStr);

                item.SubItems.Add(ThreadWorker.GetThreadTypeStr(
                    thread));

                item.SubItems.Add(
                    ThreadWorker.GetThreadedFaceTypeStr(thread));
            }

            _selecSetPopulated = (lvFeatures.Items.Count != 0);

            bOk.Enabled = ValidateOkButton();
        }

        /////////////////////////////////////////////////////////////
        // use: OnUnselect ThreadFeature Handler.
        //
        /////////////////////////////////////////////////////////////
        void InteractionManager_OnUnselectEvent(object o, 
            InteractionManager.OnUnselectEventArgs e)
        {
            foreach (System.Object obj in e.Entities)
            {
                foreach (ListViewItem item in lvFeatures.Items)
                {
                    if (item.Tag == obj)
                    {
                        item.Remove();
                        break;
                    }
                }
            }

            if (_InteractionManager.SelectedEntities.Count == 0)
                bOk.Enabled = false;
        }

        /////////////////////////////////////////////////////////////
        // Use: Displays controller form as Inventor child window.
        //
        /////////////////////////////////////////////////////////////
        public void ShowAsChild()
        {
            Show(new WindowWrapper(
                (IntPtr)_Application.MainFrameHWND));

            _InteractionManager.Initialize();

            _InteractionManager.AddSelectionFilter(
                SelectionFilterEnum.kPartFeatureFilter);

            _InteractionManager.AddPreSelectionFilter(
                ObjectTypeEnum.kThreadFeatureObject);

            _InteractionManager.DoSelect();
        }

        /////////////////////////////////////////////////////////////
        // Use: Ok Button clicked Handler
        //
        /////////////////////////////////////////////////////////////
        private void bOk_Click(object sender, EventArgs e)
        {
            bool silentOp = _Application.SilentOperation;

            _Application.SilentOperation = true;

            PartDocument template = _Application.Documents.Open(
                _ThreadTemplatePath, false) as PartDocument;

            _Application.SilentOperation = silentOp;

            if (!ValidateTemplateParameters(template))
            {
                DialogResult res = MessageBox.Show(
                    "Missing sketch parameter in template file!",
                    "Invalid Template",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);

                bOk.Enabled = false;

                _ThreadTemplatePath = string.Empty;

                tbTemplate.Text = string.Empty;

                if(template != _Application.ActiveDocument)
                    template.Close(true);

                return;
            }

            List<ThreadFeature> threads = new List<ThreadFeature>();

            foreach (System.Object selectedObj in 
                _InteractionManager.SelectedEntities)
            {
                ThreadFeature thread = selectedObj as ThreadFeature;

                if (thread.Suppressed)
                    continue;

                threads.Add(thread);
            }

            PlanarSketch templateSketch = 
                template.ComponentDefinition.Sketches[1];
           
            if (!ThreadWorker.ModelizeThreads(_Document, 
                templateSketch, threads, _extraPitch))
            {
                DialogResult res = MessageBox.Show(
                    "Failed to create CoilFeature... " + 
                    "Try with a bigger Pitch Offset value",
                    "Modelization Error", 
                    MessageBoxButtons.OKCancel, 
                    MessageBoxIcon.Error);

                switch (res)
                { 
                    case DialogResult.OK:
                        //template.Close(true);
                        return;
                    default:
                        break;
                }
            }

            // Problem: closing the template doc here 
            // will empty the undo stack (as designed)...

            //template.Close(true);

            _InteractionManager.Terminate();       
        }

        /////////////////////////////////////////////////////////////
        // Use: Returns True if template document contains required
        //      parameters to modelize thread.
        /////////////////////////////////////////////////////////////
        private bool ValidateTemplateParameters(PartDocument doc)
        { 
             if(!Toolkit.FindParameter(doc, "Pitch"))
                 return false;

            if (!Toolkit.FindParameter(doc, "ThreadOffset"))
                return false;

            if (!Toolkit.FindParameter(doc, "MajorRadius"))
                return false;

            if (!Toolkit.FindParameter(doc, "Angle"))
                return false;

            if (!Toolkit.FindParameter(doc, "MinorRadius"))
                return false;

            return true;
        }

        /////////////////////////////////////////////////////////////
        // Use: Cancel Button clicked Handler
        //
        /////////////////////////////////////////////////////////////
        private void bCancel_Click(object sender, EventArgs e)
        {
            _InteractionManager.Terminate();
        }

        /////////////////////////////////////////////////////////////
        // Use: Select template handler.
        //
        /////////////////////////////////////////////////////////////
        private void OnSelectTemplate()
        {
            string path = GetThreadTemplatePath();

            if (path != string.Empty)
            {
                _ThreadTemplatePath = path;

                tbTemplate.Text = path;
            }

            bOk.Enabled = ValidateOkButton();
                
        }
        
        private void bSelect_Click(object sender, EventArgs e)
        {
            OnSelectTemplate();
        }

        void tbTemplate_DoubleClick(object sender, EventArgs e)
        {
            OnSelectTemplate();
        }

        /////////////////////////////////////////////////////////////
        // Use: Prompts user to locate thread sketch template, if
        //      not found automatically.
        /////////////////////////////////////////////////////////////
        private string GetThreadTemplatePath()
        {
            try
            {
                string pathInit = 
                    Assembly.GetExecutingAssembly().Location;

                FileInfo fi = new FileInfo(pathInit);

                Inventor.FileDialog fileDlg = null;
                _Application.CreateFileDialog(out fileDlg);

                fileDlg.Filter = "Inventor Parts (*.ipt)|*.ipt";
                fileDlg.FilterIndex = 1;
                fileDlg.DialogTitle = "Select Thread Template";
                fileDlg.InitialDirectory = 
                    fi.DirectoryName + "\\Thread Templates";
                fileDlg.FileName = "BSW Template.ipt";
                fileDlg.MultiSelectEnabled = false;
                fileDlg.OptionsEnabled = false;
                fileDlg.CancelError = true;
                fileDlg.SuppressResolutionWarnings = true;

                fileDlg.ShowOpen();

                return fileDlg.FileName;
            }
            catch
            {
                return string.Empty;
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: Tries to locate thread sketch template in addin
        //      install directory.
        /////////////////////////////////////////////////////////////
        private string FindThreadTemplate()
        {
            string pathInit = 
                Assembly.GetExecutingAssembly().Location;

            FileInfo fi = new FileInfo(pathInit);

            string path = 
                fi.DirectoryName + 
                "\\Thread Templates\\BSW Template.ipt";

            if (System.IO.File.Exists(path))
            {
                _ThreadTemplatePath = path;
                return path;
            }

            return string.Empty;
        }

        /////////////////////////////////////////////////////////////
        // Use: Scroll control handler.
        //
        /////////////////////////////////////////////////////////////
        private void OnScroll(object sender, 
            ScrollEventArgs e)
        {
            _extraPitch = (1000 - scrollPitch.Value) * 0.01;

            tbExtraPitch.Text = _extraPitch.ToString("0.00");
        }

        /////////////////////////////////////////////////////////////
        // Use: Pitch Offset text changed Handler.
        //
        /////////////////////////////////////////////////////////////
        private void tbExtraPitch_TextChanged(object sender, 
            EventArgs e)
        {
            try
            {
                double extraPitch = double.Parse(tbExtraPitch.Text);

                if (extraPitch < 0.1 || extraPitch > 10.0)
                {
                    tbExtraPitch.ForeColor = System.Drawing.Color.Red;
                    _extraPitchValid = false;
                    bOk.Enabled = ValidateOkButton();
                    return;
                }

                _extraPitch = extraPitch;

                scrollPitch.Value = (int)(1000.0 - _extraPitch * 100.0);
                
                tbExtraPitch.ForeColor = System.Drawing.Color.Black;
                _extraPitchValid = true;

                bOk.Enabled = ValidateOkButton();
            }
            catch
            {
                tbExtraPitch.ForeColor = System.Drawing.Color.Red;
                _extraPitchValid = false;

                bOk.Enabled = ValidateOkButton();
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: Returns True if Ok Button can be enabled.
        //
        /////////////////////////////////////////////////////////////
        private bool ValidateOkButton()
        {
            return (_selecSetPopulated &&
                (_ThreadTemplatePath != string.Empty) &&
                _extraPitchValid);
        }
    }
}
