﻿////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Autodesk, Inc. All rights reserved 
// Written by Philippe Leefsma 2011 - ADN/Developer Technical Services
//
// This software is provided as is, without any warranty that it will work. You choose to use this tool at your own risk.
// Neither Autodesk nor the author Philippe Leefsma can be taken as responsible for any damage this tool can cause to 
// your data. Please always make a back up of your data prior to use this tool, as it will modify the documents involved 
// in the feature transformation.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Runtime.InteropServices;
using Microsoft.VisualBasic.Compatibility.VB6;
using Inventor;

namespace ThreadModeler
{
    /////////////////////////////////////////////////////////////////
    // Manages creation of the addin controls
    //
    /////////////////////////////////////////////////////////////////
    class ControlsManager
    {
        /////////////////////////////////////////////////////////////
        // Members
        //
        /////////////////////////////////////////////////////////////
        private Inventor.Application _Application;

        private string _addInGuid;

        private ButtonDefinition _PartMainControlButtonDef;

        private Controller _Controller;

        /////////////////////////////////////////////////////////////
        // Constructor
        //
        /////////////////////////////////////////////////////////////
        public ControlsManager(Inventor.Application Application, 
            string addInGuid, 
            bool firstTime)
        {
            _Application = Application;
          
            _addInGuid = addInGuid;

            Toolkit.Initialize(Application);

            ThreadWorker.Initialize(Application);

            CreateControls(firstTime);
        }

        /////////////////////////////////////////////////////////////
        // Use: Clean up.
        //
        /////////////////////////////////////////////////////////////
        public void Deactivate()
        {
            Marshal.ReleaseComObject(_PartMainControlButtonDef);
            _PartMainControlButtonDef = null;
        }

        /////////////////////////////////////////////////////////////
        // Performs UI Initialization Classic/Ribbon, loads resources
        //
        /////////////////////////////////////////////////////////////
        public void CreateControls(bool firstTime)
        {
            ControlDefinitions ctrlDefs = 
                _Application.CommandManager.ControlDefinitions;

            System.Drawing.Icon Icon32 = Resources.Thread_32x32;
            System.Drawing.Icon Icon16 = Resources.Thread_16x16;

            object IPictureDisp32 = Support.IconToIPicture(Icon32);
            object IPictureDisp16 = Support.IconToIPicture(Icon16);

            try
            {
                _PartMainControlButtonDef = 
                    ctrlDefs["Autodesk:AdskThreadModeler:PartMainCtrl"]
                        as ButtonDefinition;
            }
            catch
            {
                _PartMainControlButtonDef = 
                    ctrlDefs.AddButtonDefinition(
                        ControlDisplayName,         
                        "Autodesk:AdskThreadModeler:PartMainCtrl",
                        CommandTypesEnum.kEditMaskCmdType,
                        _addInGuid,
                        "Thread Modeler Control",
                        "Displays Thread Modeler main control",
                        IPictureDisp16,
                        IPictureDisp32,
                        ButtonDisplayEnum.kDisplayTextInLearningMode);

                CommandCategory cmdCat = 
                    _Application.CommandManager.CommandCategories.Add(
                    "Thread Modeler",
                    "Autodesk:DevTechToolsCategory",
                    _addInGuid);

                cmdCat.Add(_PartMainControlButtonDef);
            }

            _PartMainControlButtonDef.OnExecute += 
                new ButtonDefinitionSink_OnExecuteEventHandler(
                    PartMainControlButtonDef_OnExecute);
            
            if (firstTime)
            {
                if (IsRibbonUI())
                {
                    DoInitRibbonUI();
                }
                else
                {
                    DoInitClassicUI();
                }
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: OnExecute button handler.
        //
        /////////////////////////////////////////////////////////////
        void PartMainControlButtonDef_OnExecute(NameValueMap Context)
        {
            if (_Controller != null)
            {
                if (!_Controller.Visible)
                {
                    _Controller.Dispose();
                }
                else if(_PartMainControlButtonDef.Pressed)
                {
                    _Controller.Focus();
                    return;
                }
            }

            _Controller = new Controller(_Application, 
                _PartMainControlButtonDef);

            _Controller.ShowAsChild();
        }

        /////////////////////////////////////////////////////////////
        // Use: Determine UI Style Classic/Ribbon.
        //
        /////////////////////////////////////////////////////////////
        private bool IsRibbonUI()
        {
            //Ribbon start from R14 (Inventor 2010)
            if (_Application.SoftwareVersion.Major < 14)
                return false;

            UserInterfaceManager UIMng = 
                _Application.UserInterfaceManager;

            //UI can be set to classic, so need to double check
            if (UIMng.InterfaceStyle != 
                InterfaceStyleEnum.kRibbonInterface)
                return false;

            return true;
        }

        /////////////////////////////////////////////////////////////
        // Use: Classic UI Initialization.
        //
        /////////////////////////////////////////////////////////////
        void DoInitClassicUI()
        {
            //PMxPartStandardCmdBar
            CommandBar partCommandBar = 
                _Application.UserInterfaceManager.
                CommandBars["PMxPartFeatureCmdBar"];

            CommandBarControl partCtrl = 
                partCommandBar.Controls.AddButton(
                    _PartMainControlButtonDef, 0);

            partCtrl.GroupBegins = true;
        }

        /////////////////////////////////////////////////////////////
        // Use: Ribbon UI Intitialization.
        //
        /////////////////////////////////////////////////////////////
        void DoInitRibbonUI()
        {
            Ribbon partRibbon = 
                _Application.UserInterfaceManager.Ribbons["Part"];

            RibbonTab partTab = 
                partRibbon.RibbonTabs["id_TabTools"];

            RibbonPanel partPanel = 
                partTab.RibbonPanels.Add(
                    "Thread Modeler",
                    "Autodesk:AdskThreadModeler:PartPanel",
                    _addInGuid,
                    "", false);

            partPanel.CommandControls.AddButton(
                _PartMainControlButtonDef, 
                true, true, "", false);
        }

        /////////////////////////////////////////////////////////////
        // Use: Determine name of the Button inside UI.
        //
        /////////////////////////////////////////////////////////////
        private string ControlDisplayName
        {
            get
            {
                return (IsRibbonUI() ? 
                    "Controller" : "Thread Modeler");
            }
        }
    }
}
