﻿////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Autodesk, Inc. All rights reserved 
// Written by Philippe Leefsma 2011 - ADN/Developer Technical Services
//
// This software is provided as is, without any warranty that it will work. You choose to use this tool at your own risk.
// Neither Autodesk nor the author Philippe Leefsma can be taken as responsible for any damage this tool can cause to 
// your data. Please always make a back up of your data prior to use this tool, as it will modify the documents involved 
// in the feature transformation.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Inventor;

namespace ThreadModeler
{
    /////////////////////////////////////////////////////////////////
    // A wrapper class to handle Interaction Events
    //
    /////////////////////////////////////////////////////////////////
    class InteractionManager
    {
        /////////////////////////////////////////////////////////////
        // Members
        //
        /////////////////////////////////////////////////////////////
        private Inventor.Application _Application;

        private InteractionEvents _InteractionEvents;
        private SelectEvents _SelectEvents;

        private bool _AllowCleanup;

        private List<System.Object> _SelectedEntities;

        private List<ObjectTypeEnum> _PreSelectFilters;

        /////////////////////////////////////////////////////////////
        // Constructor
        //
        /////////////////////////////////////////////////////////////
        public InteractionManager(Inventor.Application Application)
        {
            _Application = Application;

            _InteractionEvents = null;

            _SelectedEntities = new List<System.Object>();

            _PreSelectFilters = new List<ObjectTypeEnum>();
        }

        /////////////////////////////////////////////////////////////
        // Use: Returns list of currently selected entities
        //
        /////////////////////////////////////////////////////////////
        public List<System.Object> SelectedEntities
        {
            get
            {
                return _SelectedEntities;
            }
        }

        /////////////////////////////////////////////////////////////
        // use: Initializes event handlers
        //
        /////////////////////////////////////////////////////////////
        public void Initialize()
        {
            _InteractionEvents = 
               _Application.CommandManager.CreateInteractionEvents();

            _SelectEvents = _InteractionEvents.SelectEvents;

            _SelectEvents.Enabled = true;
            _SelectEvents.SingleSelectEnabled = false;

            _SelectEvents.OnPreSelect += 
                new SelectEventsSink_OnPreSelectEventHandler(
                    SelectEvents_OnPreSelect);

            _SelectEvents.OnSelect +=
                new SelectEventsSink_OnSelectEventHandler(
                    SelectEvents_OnSelect);

            _SelectEvents.OnUnSelect += 
                new SelectEventsSink_OnUnSelectEventHandler(
                    SelectEvents_OnUnSelect);

            _InteractionEvents.OnTerminate +=
                new InteractionEventsSink_OnTerminateEventHandler(
                    InteractionEvents_OnTerminate);
        }

        /////////////////////////////////////////////////////////////
        // Use: Add selection filter.
        //
        /////////////////////////////////////////////////////////////
        public void AddSelectionFilter(SelectionFilterEnum filter)
        {
            if (_SelectEvents != null)
            {
                _SelectEvents.AddSelectionFilter(filter);
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: Add pre-selection filter.
        //
        /////////////////////////////////////////////////////////////
        public void AddPreSelectionFilter(
            ObjectTypeEnum filter2nd)
        {
            _PreSelectFilters.Add(filter2nd);
        }

        /////////////////////////////////////////////////////////////
        // Use: Starts selection.
        //
        /////////////////////////////////////////////////////////////
        public void DoSelect()
        {
            _AllowCleanup = false;

            _InteractionEvents.Stop();

            System.Windows.Forms.Application.DoEvents();

            _SelectedEntities.Clear();

            _InteractionEvents.Start();

            _AllowCleanup = true;
        }

        /////////////////////////////////////////////////////////////
        // Use: OnPreSelect Handler.
        //
        /////////////////////////////////////////////////////////////
        private void SelectEvents_OnPreSelect(
            ref object PreSelectEntity, 
            out bool DoHighlight, 
            ref ObjectCollection MorePreSelectEntities, 
            SelectionDeviceEnum SelectionDevice, 
            Point ModelPosition, 
            Point2d ViewPosition, 
            View View)
        {
            if (_PreSelectFilters.Count != 0 && 
                !_PreSelectFilters.Contains(
                    Toolkit.GetInventorType(PreSelectEntity)))
            {
                DoHighlight = false;
                return;
            }

            DoHighlight = true;
        }

        /////////////////////////////////////////////////////////////
        // Use: OnSelect Handler.
        //
        /////////////////////////////////////////////////////////////
        private void SelectEvents_OnSelect(
            ObjectsEnumerator JustSelectedEntities,
            SelectionDeviceEnum SelectionDevice,
            Point ModelPosition,
            Point2d ViewPosition,
            View View)
        {
            if (JustSelectedEntities.Count != 0)
            {
                System.Object selectedObj = JustSelectedEntities[1];

                _SelectedEntities.Add(selectedObj);

                OnSelect(new OnSelectEventArgs(JustSelectedEntities));
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: OnUnSelect Handler.
        //
        /////////////////////////////////////////////////////////////
        void SelectEvents_OnUnSelect(
            ObjectsEnumerator UnSelectedEntities,
            SelectionDeviceEnum SelectionDevice,
            Point ModelPosition,
            Point2d ViewPosition,
            View View)
        {
            foreach (System.Object unselectedObj in 
                UnSelectedEntities)
            {
                if (_SelectedEntities.Contains(unselectedObj))
                {
                    _SelectedEntities.Remove(unselectedObj);   
                }
            }

            OnUnselect(new OnUnselectEventArgs(
                UnSelectedEntities));
        }

        /////////////////////////////////////////////////////////////
        // Use: Performs interaction cleanup.
        //
        /////////////////////////////////////////////////////////////
        private void CleanUp()
        {
            if (_InteractionEvents != null)
            {
                _SelectedEntities.Clear();

                _PreSelectFilters.Clear();

                _SelectEvents.OnPreSelect -=
                   new SelectEventsSink_OnPreSelectEventHandler(
                       SelectEvents_OnPreSelect);

                _SelectEvents.OnSelect -=
                    new SelectEventsSink_OnSelectEventHandler(
                        SelectEvents_OnSelect);

                _SelectEvents.OnUnSelect -=
                   new SelectEventsSink_OnUnSelectEventHandler(
                       SelectEvents_OnUnSelect);

                _InteractionEvents.OnTerminate -=
                    new InteractionEventsSink_OnTerminateEventHandler(
                        InteractionEvents_OnTerminate);

                Marshal.ReleaseComObject(_SelectEvents);
                _SelectEvents = null;

                Marshal.ReleaseComObject(_InteractionEvents);
                _InteractionEvents = null;
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: Terminates Interaction event.
        //
        /////////////////////////////////////////////////////////////
        public void Terminate()
        {
            if (_InteractionEvents != null)
            {
                _AllowCleanup = true;

                _InteractionEvents.Stop();
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: OnTerminate handler.
        //
        /////////////////////////////////////////////////////////////
        private void InteractionEvents_OnTerminate()
        {
            if (_AllowCleanup)
            {
                CleanUp();

                OnTerminate(new OnTerminateEventArgs());
            }
        }

        /////////////////////////////////////////////////////////////
        // Use: Exposes OnTerminate Event for InteractionManager.
        //
        /////////////////////////////////////////////////////////////
        public class OnTerminateEventArgs : EventArgs
        {
            public OnTerminateEventArgs()
            {

            }
        }

        private void OnTerminate(OnTerminateEventArgs e)
        {
            if (OnTerminateEvent != null)
                OnTerminateEvent(this, e);
        }

        public delegate void OnTerminateHandler(object o, 
            OnTerminateEventArgs e);

        public event OnTerminateHandler OnTerminateEvent = null;

        /////////////////////////////////////////////////////////////
        // Use: Exposes OnSelect Event for InteractionManager.
        //
        /////////////////////////////////////////////////////////////
        public class OnSelectEventArgs : EventArgs
        {
            private ObjectsEnumerator _Entities;

            public OnSelectEventArgs(ObjectsEnumerator entities)
            {
                _Entities = entities;
            }

            public ObjectsEnumerator Entities
            {
                get 
                {
                    return _Entities;
                }
            }
        }

        private void OnSelect(OnSelectEventArgs e)
        {
            if (OnSelectEvent != null)
                OnSelectEvent(this, e);
        }

        public delegate void OnSelectHandler(object o, 
            OnSelectEventArgs e);

        public event OnSelectHandler OnSelectEvent = null;

        /////////////////////////////////////////////////////////////
        // Use: Exposes OnUnselect Event for InteractionManager.
        //
        /////////////////////////////////////////////////////////////
        public class OnUnselectEventArgs : EventArgs
        {
           private ObjectsEnumerator _Entities;

           public OnUnselectEventArgs(ObjectsEnumerator entities)
            {
                _Entities = entities;
            }

            public ObjectsEnumerator Entities
            {
                get 
                {
                    return _Entities;
                }
            }
        }

        private void OnUnselect(OnUnselectEventArgs e)
        {
            if (OnUnselectEvent != null)
                OnUnselectEvent(this, e);
        }

        public delegate void OnUnselectHandler(object o, 
            OnUnselectEventArgs e);

        public event OnUnselectHandler OnUnselectEvent = null;
    }
}
